"""Unicode Properties from Unicode version 16.0.0 (autogen)."""
from .age import *  # noqa
from .alias import *  # noqa
from .bidiclass import *  # noqa
from .binary import *  # noqa
from .block import *  # noqa
from .bidipairedbrackettype import *  # noqa
from .canonicalcombiningclass import *  # noqa
from .decompositiontype import *  # noqa
from .eastasianwidth import *  # noqa
from .generalcategory import *  # noqa
from .graphemeclusterbreak import *  # noqa
from .hangulsyllabletype import *  # noqa
from .indicpositionalcategory import *  # noqa
from .indicsyllabiccategory import *  # noqa
from .joininggroup import *  # noqa
from .joiningtype import *  # noqa
from .linebreak import *  # noqa
from .numerictype import *  # noqa
from .numericvalue import *  # noqa
from .quickcheck import *  # noqa
from .sentencebreak import *  # noqa
from .script import *  # noqa
from .scriptextensions import *  # noqa
from .verticalorientation import *  # noqa
from .wordbreak import *  # noqa



"""Typing objects for the Regex library."""
import regex  # type: ignore[import]
from typing import _alias  # type: ignore[attr-defined]

Pattern = _alias(type(regex.compile('')), 1)
Match = _alias(type(regex.compile('').match('')), 1)



r"""
Backrefs re.

Add the ability to use the following backrefs with re:

 - `\l`                                                          - Lowercase character class (search)
 - `\c`                                                          - Uppercase character class (search)
 - `\L`                                                          - Inverse of lowercase character class (search)
 - `\C`                                                          - Inverse of uppercase character class (search)
 - `\Q` and `\Q...\E`                                            - Escape/quote chars (search)
 - `\c` and `\C...\E`                                            - Uppercase char or chars (replace)
 - `\l` and `\L...\E`                                            - Lowercase char or chars (replace)
 - `[:ascii:]`                                                   - Posix style classes (search)
 - `[:^ascii:]`                                                  - Inverse Posix style classes (search)
 - `\pL`, `\p{Lu}`, \p{Letter}, `\p{gc=Uppercase_Letter}`        - Unicode properties (search Unicode)
 - `\PL`, `\P{Lu}`, `\p{^Lu}`                                    - Inverse Unicode properties (search Unicode)
 - `\N{Black Club Suit}`                                         - Unicode character by name (search & replace)
 - `\u0000` and `\U00000000`                                     - Unicode characters (replace)
 - `\m`                                                          - Starting word boundary (search)
 - `\M`                                                          - Ending word boundary (search)
 - `\R`                                                          - Generic line breaks (search)
 - `\X`                                                          - Simplified grapheme clusters (search)

Licensed under MIT
Copyright (c) 2011 - 2020 Isaac Muse <isaacmuse@gmail.com>
"""
from __future__ import annotations
import re as _re
import copyreg as _copyreg
from functools import lru_cache as _lru_cache
from . import util as _util
from . import _bre_parse
from ._bre_parse import ReplaceTemplate
from typing import AnyStr, Pattern, Match, Callable, Any, Generic, Mapping, Iterator, cast

__all__ = (
    "expand", "expandf", "search", "match", "fullmatch", "split", "findall", "finditer", "sub", "subf",
    "subn", "subfn", "purge", "escape", "fullmatch", "DEBUG", "I", "IGNORECASE", "L", "LOCALE", "M", "MULTILINE",
    "S", "DOTALL", "U", "UNICODE", "X", "VERBOSE", "compile", "compile_search", "compile_replace", "Bre",
    "ReplaceTemplate", "A", "ASCII"
)

# Expose some common re flags and methods to
# save having to import re and backrefs libraries
DEBUG = _re.DEBUG
I = _re.I
IGNORECASE = _re.IGNORECASE
L = _re.L
LOCALE = _re.LOCALE
M = _re.M
MULTILINE = _re.MULTILINE
S = _re.S
DOTALL = _re.DOTALL
U = _re.U
UNICODE = _re.UNICODE
X = _re.X
VERBOSE = _re.VERBOSE
A = _re.A
ASCII = _re.ASCII
escape = _re.escape

# Replace flags
FORMAT = 1

# Maximum size of the cache.
_MAXCACHE = 500

_RE_TYPE = type(_re.compile('', 0))


@_lru_cache(maxsize=_MAXCACHE)
def _cached_search_compile(
    pattern: AnyStr,
    re_verbose: bool,
    re_unicode: bool,
    pattern_type: type[AnyStr]
) -> AnyStr:
    """Cached search compile."""

    return _bre_parse._SearchParser(pattern, re_verbose, re_unicode).parse()


@_lru_cache(maxsize=_MAXCACHE)
def _cached_replace_compile(
    pattern: Pattern[AnyStr],
    repl: AnyStr,
    flags: int,
    pattern_type: type[AnyStr]
) -> ReplaceTemplate[AnyStr]:
    """Cached replace compile."""

    return _bre_parse._ReplaceParser(pattern, repl, bool(flags & FORMAT)).parse()


def _get_cache_size(replace: bool = False) -> int:
    """Get size of cache."""

    if not replace:
        size = _cached_search_compile.cache_info().currsize
    else:
        size = _cached_replace_compile.cache_info().currsize
    return size


def _purge_cache() -> None:
    """Purge the cache."""

    _cached_replace_compile.cache_clear()
    _cached_search_compile.cache_clear()


def _is_replace(obj: Any) -> bool:
    """Check if object is a replace object."""

    return isinstance(obj, ReplaceTemplate)


def _apply_replace_backrefs(
    m: Match[AnyStr] | None,
    repl: ReplaceTemplate[AnyStr] | AnyStr,
    flags: int = 0
) -> AnyStr:
    """Expand with either the `ReplaceTemplate` or compile on the fly, or return None."""

    if m is None:
        raise ValueError("Match is None!")

    if isinstance(repl, ReplaceTemplate):
        return repl.expand(m)
    return _bre_parse._ReplaceParser(m.re, repl, bool(flags & FORMAT)).parse().expand(m)


def _apply_search_backrefs(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    flags: int = 0
) -> AnyStr | Pattern[AnyStr]:
    """Apply the search backrefs to the search pattern."""

    if isinstance(pattern, (str, bytes)):
        re_verbose = bool(VERBOSE & flags)
        re_unicode = None
        if bool((ASCII | LOCALE) & flags):
            re_unicode = False
        elif bool(UNICODE & flags):
            re_unicode = True
        if not (flags & DEBUG):
            p = _cached_search_compile(
                pattern, re_verbose, re_unicode, type(pattern)
            )  # type: AnyStr | Pattern[AnyStr]
        else:  # pragma: no cover
            p = _bre_parse._SearchParser(pattern, re_verbose, re_unicode).parse()
    elif isinstance(pattern, Bre):
        if flags:
            raise ValueError("Cannot process flags argument with a compiled pattern")
        p = pattern._pattern
    elif isinstance(pattern, _RE_TYPE):
        if flags:
            raise ValueError("Cannot process flags argument with a compiled pattern!")
        p = pattern
    else:
        raise TypeError("Not a string or compiled pattern!")
    return p


def _assert_expandable(repl: Any, use_format: bool = False) -> None:
    """Check if replace template is expandable."""

    if isinstance(repl, ReplaceTemplate):
        if repl.use_format != use_format:
            if use_format:
                raise ValueError("Replace not compiled as a format replace")
            else:
                raise ValueError("Replace should not be compiled as a format replace!")
    elif not isinstance(repl, (str, bytes)):
        raise TypeError("Expected string, buffer, or compiled replace!")


###########################
# API
##########################
class Bre(_util.Immutable, Generic[AnyStr]):
    """Bre object."""

    _pattern: Pattern[AnyStr]
    auto_compile: bool
    _hash: int

    __slots__ = ("_pattern", "auto_compile", "_hash")

    def __init__(self, pattern: Pattern[AnyStr], auto_compile: bool = True) -> None:
        """Initialization."""

        super().__init__(
            _pattern=pattern,
            auto_compile=auto_compile,
            _hash=hash((type(self), type(pattern), pattern, auto_compile))
        )

    @property
    def pattern(self) -> AnyStr:
        """Return pattern."""

        return self._pattern.pattern

    @property
    def flags(self) -> int:
        """Return flags."""

        return self._pattern.flags

    @property
    def groupindex(self) -> Mapping[str, int]:
        """Return group index."""

        return self._pattern.groupindex

    @property
    def groups(self) -> tuple[AnyStr | None, ...]:
        """Return groups."""

        return cast('tuple[AnyStr | None, ...]', self._pattern.groups)

    @property
    def scanner(self) -> Any:
        """Return scanner."""

        return self._pattern.scanner  # type: ignore[attr-defined]

    def __hash__(self) -> int:
        """Hash."""

        return self._hash

    def __eq__(self, other: Any) -> bool:
        """Equal."""

        return (
            isinstance(other, Bre) and
            self._pattern == other._pattern and
            self.auto_compile == other.auto_compile
        )

    def __ne__(self, other: Any) -> bool:
        """Equal."""

        return (
            not isinstance(other, Bre) or
            self._pattern != other._pattern or
            self.auto_compile != other.auto_compile
        )

    def __repr__(self) -> str:  # pragma: no cover
        """Representation."""

        return '{}.{}({!r}, auto_compile={!r})'.format(
            self.__module__, self.__class__.__name__, self._pattern, self.auto_compile
        )

    def _auto_compile(
        self,
        template: AnyStr | Callable[..., AnyStr],
        use_format: bool = False
    ) -> AnyStr | Callable[..., AnyStr]:
        """Compile replacements."""

        if isinstance(template, ReplaceTemplate):
            if use_format != template.use_format:
                raise ValueError("Compiled replace cannot be a format object!")
        elif isinstance(template, ReplaceTemplate) or (isinstance(template, (str, bytes)) and self.auto_compile):
            return self.compile(template, (FORMAT if use_format and not isinstance(template, ReplaceTemplate) else 0))
        elif isinstance(template, (str, bytes)) and use_format:
            # Reject an attempt to run format replace when auto-compiling
            # of template strings has been disabled and we are using a
            # template string.
            raise AttributeError('Format replaces cannot be called without compiling replace template!')
        return template

    def compile(  # noqa A001
        self,
        repl: AnyStr | Callable[..., AnyStr],
        flags: int = 0
    ) -> Callable[..., AnyStr]:
        """Compile replace."""

        return compile_replace(self._pattern, repl, flags)

    def search(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> Match[AnyStr] | None:
        """Apply `search`."""

        return self._pattern.search(string, *args, **kwargs)

    def match(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> Match[AnyStr] | None:
        """Apply `match`."""

        return self._pattern.match(string, *args, **kwargs)

    def fullmatch(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> Match[AnyStr] | None:
        """Apply `fullmatch`."""

        return self._pattern.fullmatch(string, *args, **kwargs)

    def split(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> list[AnyStr]:
        """Apply `split`."""

        return self._pattern.split(string, *args, **kwargs)

    def findall(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> list[AnyStr] | list[tuple[AnyStr, ...]]:
        """Apply `findall`."""

        return self._pattern.findall(string, *args, **kwargs)

    def finditer(
        self,
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> Iterator[Match[AnyStr]]:
        """Apply `finditer`."""

        return self._pattern.finditer(string, *args, **kwargs)

    def sub(
        self,
        repl: AnyStr | Callable[..., AnyStr],
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> AnyStr:
        """Apply `sub`."""

        return self._pattern.sub(self._auto_compile(repl), string, *args, **kwargs)

    def subf(  # noqa A002
        self,
        repl: AnyStr | Callable[..., AnyStr],
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> AnyStr:
        """Apply `sub` with format style replace."""

        return self._pattern.sub(self._auto_compile(repl, True), string, *args, **kwargs)

    def subn(
        self,
        repl: AnyStr | Callable[..., AnyStr],
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> tuple[AnyStr, int]:
        """Apply `subn` with format style replace."""

        return self._pattern.subn(self._auto_compile(repl), string, *args, **kwargs)

    def subfn(  # noqa A002
        self,
        repl: AnyStr | Callable[..., AnyStr],
        string: AnyStr,
        *args: Any,
        **kwargs: Any
    ) -> tuple[AnyStr, int]:
        """Apply `subn` after applying backrefs."""

        return self._pattern.subn(self._auto_compile(repl, True), string, *args, **kwargs)


def compile(  # noqa A001
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    flags: int = 0,
    auto_compile: bool | None = None
) -> Bre[AnyStr]:
    """Compile both the search or search and replace into one object."""

    if isinstance(pattern, Bre):
        if auto_compile is not None:
            raise ValueError("Cannot compile Bre with a different auto_compile!")
        elif flags != 0:
            raise ValueError("Cannot process flags argument with a compiled pattern")
        return pattern
    else:
        if auto_compile is None:
            auto_compile = True

        return Bre(compile_search(pattern, flags), auto_compile)


def compile_search(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    flags: int = 0
) -> Pattern[AnyStr]:
    """Compile with extended search references."""

    return _re.compile(_apply_search_backrefs(pattern, flags), flags)


def compile_replace(
    pattern: Pattern[AnyStr],
    repl: AnyStr | Callable[..., AnyStr],
    flags: int = 0
) -> Callable[..., AnyStr]:
    """Construct a method that can be used as a replace method for `sub`, `subn`, etc."""

    if pattern is not None and isinstance(pattern, _RE_TYPE):
        if isinstance(repl, (str, bytes)):
            if not (pattern.flags & DEBUG):
                call = _cached_replace_compile(pattern, repl, flags, type(repl))
            else:  # pragma: no cover
                call = _bre_parse._ReplaceParser(pattern, repl, bool(flags & FORMAT)).parse()
        elif isinstance(repl, ReplaceTemplate):
            if flags:
                raise ValueError("Cannot process flags argument with a ReplaceTemplate!")
            if repl.pattern_hash != hash(pattern):
                raise ValueError("Pattern hash doesn't match hash in compiled replace!")
            call = repl
        else:
            raise TypeError("Not a valid type!")
    else:
        raise TypeError("Pattern must be a compiled regular expression!")
    return call


def purge() -> None:
    """Purge caches."""

    _purge_cache()
    _re.purge()


def expand(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
    """Expand the string using the replace pattern or function."""

    _assert_expandable(repl)
    return _apply_replace_backrefs(m, repl)


def expandf(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
    """Expand the string using the format replace pattern or function."""

    _assert_expandable(repl, True)
    return _apply_replace_backrefs(m, repl, flags=FORMAT)


def search(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    flags: int | _re.RegexFlag = 0,
    *args: Any,
    **kwargs: Any
) -> Match[AnyStr] | None:
    """Apply `search` after applying backrefs."""

    return _re.search(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)


def prefixmatch(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> Match[AnyStr] | None:
    """Apply `match` after applying backrefs."""

    return _re.match(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)


match = prefixmatch


def fullmatch(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> Match[AnyStr] | None:
    """Apply `fullmatch` after applying backrefs."""

    return _re.fullmatch(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)


def split(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    maxsplit: int = 0,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> list[AnyStr]:
    """Apply `split` after applying backrefs."""

    return _re.split(
        _apply_search_backrefs(pattern, flags),
        string,
        maxsplit=maxsplit,
        flags=flags,
        **kwargs
    )


def findall(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> list[AnyStr] | list[tuple[AnyStr, ...]]:
    """Apply `findall` after applying backrefs."""

    return _re.findall(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)


def finditer(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    string: AnyStr,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> Iterator[Match[AnyStr]]:
    """Apply `finditer` after applying backrefs."""

    return _re.finditer(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)


def sub(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    repl: AnyStr | Callable[..., AnyStr],
    string: AnyStr,
    count: int = 0,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> AnyStr:
    """Apply `sub` after applying backrefs."""

    is_replace = _is_replace(repl)
    is_string = isinstance(repl, (str, bytes))
    if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
        raise ValueError("Compiled replace cannot be a format object!")

    pattern = compile_search(pattern, flags)
    return _re.sub(
        pattern,
        (compile_replace(pattern, repl) if is_replace or is_string else repl),
        string,
        count=count,
        flags=0,
        **kwargs
    )


def subf(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    repl: AnyStr | Callable[..., AnyStr],
    string: AnyStr,
    count: int = 0,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> AnyStr:
    """Apply `sub` with format style replace."""

    is_replace = _is_replace(repl)
    is_string = isinstance(repl, (str, bytes))
    if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
        raise ValueError("Compiled replace is not a format object!")

    pattern = compile_search(pattern, flags)
    rflags = FORMAT if is_string else 0
    return _re.sub(
        pattern,
        (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl),
        string,
        count=count,
        flags=0,
        **kwargs
    )


def subn(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    repl: AnyStr | Callable[..., AnyStr],
    string: AnyStr,
    count: int = 0,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> tuple[AnyStr, int]:
    """Apply `subn` with format style replace."""

    is_replace = _is_replace(repl)
    is_string = isinstance(repl, (str, bytes))
    if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
        raise ValueError("Compiled replace cannot be a format object!")

    pattern = compile_search(pattern, flags)
    return _re.subn(
        pattern,
        (compile_replace(pattern, repl) if is_replace or is_string else repl),
        string,
        count=count,
        flags=0,
        **kwargs
    )


def subfn(
    pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
    repl: AnyStr | Callable[..., AnyStr],
    string: AnyStr,
    count: int = 0,
    flags: int | _re.RegexFlag = 0,
    **kwargs: Any
) -> tuple[AnyStr, int]:
    """Apply `subn` after applying backrefs."""

    is_replace = _is_replace(repl)
    is_string = isinstance(repl, (str, bytes))
    if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
        raise ValueError("Compiled replace is not a format object!")

    pattern = compile_search(pattern, flags)
    rflags = FORMAT if is_string else 0
    return _re.subn(
        pattern,
        (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl),
        string,
        count=count,
        flags=0,
        **kwargs
    )


def _pickle(p):  # type: ignore[no-untyped-def]
    return Bre, (p._pattern, p.auto_compile)


_copyreg.pickle(Bre, _pickle)



"""
Utilities and compatibility abstraction.

Licensed under MIT
Copyright (c) 2015 - 2020 Isaac Muse <isaacmuse@gmail.com>
"""
from __future__ import annotations
import warnings
import sys
from typing import Any, Callable, AnyStr

PY311 = (3, 11) <= sys.version_info
PY312 = (3, 12) <= sys.version_info

FMT_FIELD = 0
FMT_INDEX = 1
FMT_ATTR = 2
FMT_CONV = 3
FMT_SPEC = 4


class StringIter:
    """Preprocess replace tokens."""

    def __init__(self, text: str) -> None:
        """Initialize."""

        self._string = text
        self._index = 0

    def __iter__(self) -> StringIter:
        """Iterate."""

        return self

    def __next__(self) -> str:
        """Python 3 iterator compatible next."""

        return self.iternext()

    @property
    def index(self) -> int:
        """Get Index."""

        return self._index

    def rewind(self, count: int) -> None:
        """Rewind index."""

        if count > self._index:  # pragma: no cover
            raise ValueError("Can't rewind past beginning!")

        self._index -= count

    def iternext(self) -> str:
        """Iterate through characters of the string."""

        try:
            char = self._string[self._index]
            self._index += 1
        except IndexError as e:
            raise StopIteration from e

        return char


def _to_bstr(obj: Any) -> bytes:
    """Convert to byte string."""

    if isinstance(obj, str):
        return obj.encode('ascii', 'backslashreplace')
    elif not isinstance(obj, bytes):
        return str(obj).encode('ascii', 'backslashreplace')
    return obj


def _to_str(obj: Any) -> str:
    """Convert to string."""

    if not isinstance(obj, str):
        return str(obj)
    return obj


def format_captures(
    captures: list[AnyStr],
    formatting: tuple[tuple[int, Any]],
    converter: Callable[[Any], AnyStr],
    default: AnyStr
) -> AnyStr:
    """Perform a string format on a set of captures."""

    capture = captures  # type: Any
    for i, fmt in enumerate(formatting, 0):
        if i == 0:
            continue
        fmt_type, value = fmt
        if fmt_type == FMT_ATTR:
            # Attribute
            capture = getattr(capture, value)
        elif fmt_type == FMT_INDEX:
            # Index
            if value is not None:
                capture = capture[value]
            else:
                capture = default if not capture else capture[0]
        elif fmt_type == FMT_CONV:
            # Conversion
            if value == 'a':
                capture = ascii(capture)
            elif value == 'r':
                capture = repr(capture)
            elif value == 's':
                # If the object is not string or byte string already
                capture = str(capture)
        elif fmt_type == FMT_SPEC:
            # Integers and floats don't have an explicit 's' format type.
            if value[3] and value[3] == 's':
                if isinstance(capture, int):  # pragma: no cover
                    raise ValueError("Unknown format code 's' for object of type 'int'")
                if isinstance(capture, float):  # pragma: no cover
                    raise ValueError("Unknown format code 's' for object of type 'float'")

            # Ensure object is a byte string
            capture = converter(capture)

            spec_type = value[1]
            if spec_type == '^':
                capture = capture.center(value[2], value[0])
            elif spec_type == ">":
                capture = capture.rjust(value[2], value[0])
            else:
                capture = capture.ljust(value[2], value[0])

    # Make sure the final object is a byte string
    return converter(capture)


class Immutable:
    """Immutable."""

    __slots__: tuple[Any, ...] = ()

    def __init__(self, **kwargs: Any) -> None:
        """Initialize."""

        for k, v in kwargs.items():
            super().__setattr__(k, v)

    def __setattr__(self, name: str, value: Any) -> None:
        """Prevent mutability."""

        raise AttributeError('Class is immutable!')


def warn_deprecated(message: str, stacklevel: int = 2) -> None:  # pragma: no cover
    """Warn deprecated."""

    warnings.warn(
        message,
        category=DeprecationWarning,
        stacklevel=stacklevel
    )



"""Unicode Properties from Unicode version 16.0.0 (autogen)."""
from __future__ import annotations

unicode_alias: dict[str, dict[str, str]] = {
    "_": {
        "age": "age",
        "bc": "bidiclass",
        "blk": "block",
        "bpt": "bidipairedbrackettype",
        "ccc": "canonicalcombiningclass",
        "dt": "decompositiontype",
        "ea": "eastasianwidth",
        "gc": "generalcategory",
        "gcb": "graphemeclusterbreak",
        "hst": "hangulsyllabletype",
        "indicconjunctbreak": "incb",
        "inpc": "indicpositionalcategory",
        "insc": "indicsyllabiccategory",
        "jg": "joininggroup",
        "jt": "joiningtype",
        "lb": "linebreak",
        "nfcqc": "nfcquickcheck",
        "nfdqc": "nfdquickcheck",
        "nfkcqc": "nfkcquickcheck",
        "nfkdqc": "nfkdquickcheck",
        "nt": "numerictype",
        "nv": "numericvalue",
        "sb": "sentencebreak",
        "sc": "script",
        "scx": "scriptextensions",
        "vo": "verticalorientation",
        "wb": "wordbreak"
    },
    "age": {
        "unassigned": "na",
        "v100": "10.0",
        "v11": "1.1",
        "v110": "11.0",
        "v120": "12.0",
        "v121": "12.1",
        "v130": "13.0",
        "v140": "14.0",
        "v150": "15.0",
        "v151": "15.1",
        "v160": "16.0",
        "v20": "2.0",
        "v21": "2.1",
        "v30": "3.0",
        "v31": "3.1",
        "v32": "3.2",
        "v40": "4.0",
        "v41": "4.1",
        "v50": "5.0",
        "v51": "5.1",
        "v52": "5.2",
        "v60": "6.0",
        "v61": "6.1",
        "v62": "6.2",
        "v63": "6.3",
        "v70": "7.0",
        "v80": "8.0",
        "v90": "9.0"
    },
    "bidiclass": {
        "arabicletter": "al",
        "arabicnumber": "an",
        "boundaryneutral": "bn",
        "commonseparator": "cs",
        "europeannumber": "en",
        "europeanseparator": "es",
        "europeanterminator": "et",
        "firststrongisolate": "fsi",
        "lefttoright": "l",
        "lefttorightembedding": "lre",
        "lefttorightisolate": "lri",
        "lefttorightoverride": "lro",
        "nonspacingmark": "nsm",
        "otherneutral": "on",
        "paragraphseparator": "b",
        "popdirectionalformat": "pdf",
        "popdirectionalisolate": "pdi",
        "righttoleft": "r",
        "righttoleftembedding": "rle",
        "righttoleftisolate": "rli",
        "righttoleftoverride": "rlo",
        "segmentseparator": "s",
        "whitespace": "ws"
    },
    "bidipairedbrackettype": {
        "close": "c",
        "none": "n",
        "open": "o"
    },
    "binary": {
        "ahex": "asciihexdigit",
        "alpha": "alphabetic",
        "bidic": "bidicontrol",
        "bidim": "bidimirrored",
        "blank": "posixblank",
        "cased": "cased",
        "ce": "compositionexclusion",
        "ci": "caseignorable",
        "compex": "fullcompositionexclusion",
        "cwcf": "changeswhencasefolded",
        "cwcm": "changeswhencasemapped",
        "cwkcf": "changeswhennfkccasefolded",
        "cwl": "changeswhenlowercased",
        "cwt": "changeswhentitlecased",
        "cwu": "changeswhenuppercased",
        "dash": "dash",
        "dep": "deprecated",
        "di": "defaultignorablecodepoint",
        "dia": "diacritic",
        "ebase": "emojimodifierbase",
        "ecomp": "emojicomponent",
        "emod": "emojimodifier",
        "emoji": "emoji",
        "epres": "emojipresentation",
        "ext": "extender",
        "extpict": "extendedpictographic",
        "graph": "posixgraph",
        "grbase": "graphemebase",
        "grext": "graphemeextend",
        "grlink": "graphemelink",
        "h": "horizspace",
        "hex": "hexdigit",
        "hyphen": "hyphen",
        "idc": "idcontinue",
        "idcompatmathcontinue": "idcompatmathcontinue",
        "idcompatmathstart": "idcompatmathstart",
        "ideo": "ideographic",
        "ids": "idstart",
        "idsb": "idsbinaryoperator",
        "idst": "idstrinaryoperator",
        "idsu": "idsunaryoperator",
        "joinc": "joincontrol",
        "loe": "logicalorderexception",
        "lower": "lowercase",
        "math": "math",
        "mcm": "modifiercombiningmark",
        "nchar": "noncharactercodepoint",
        "oalpha": "otheralphabetic",
        "odi": "otherdefaultignorablecodepoint",
        "ogrext": "othergraphemeextend",
        "oidc": "otheridcontinue",
        "oids": "otheridstart",
        "olower": "otherlowercase",
        "omath": "othermath",
        "oupper": "otheruppercase",
        "patsyn": "patternsyntax",
        "patws": "patternwhitespace",
        "pcm": "prependedconcatenationmark",
        "posixalpha": "alphabetic",
        "posixlower": "lowercase",
        "posixspace": "whitespace",
        "posixupper": "uppercase",
        "print": "posixprint",
        "qmark": "quotationmark",
        "radical": "radical",
        "ri": "regionalindicator",
        "sd": "softdotted",
        "space": "whitespace",
        "sterm": "sentenceterminal",
        "term": "terminalpunctuation",
        "uideo": "unifiedideograph",
        "upper": "uppercase",
        "v": "vertspace",
        "vs": "variationselector",
        "word": "posixword",
        "wspace": "whitespace",
        "xidc": "xidcontinue",
        "xids": "xidstart"
    },
    "block": {
        "alchemical": "alchemicalsymbols",
        "alphabeticpf": "alphabeticpresentationforms",
        "ancientgreekmusic": "ancientgreekmusicalnotation",
        "arabicexta": "arabicextendeda",
        "arabicextb": "arabicextendedb",
        "arabicextc": "arabicextendedc",
        "arabicmath": "arabicmathematicalalphabeticsymbols",
        "arabicpfa": "arabicpresentationformsa",
        "arabicpfb": "arabicpresentationformsb",
        "arabicsup": "arabicsupplement",
        "ascii": "basiclatin",
        "bamumsup": "bamumsupplement",
        "bopomofoext": "bopomofoextended",
        "braille": "braillepatterns",
        "byzantinemusic": "byzantinemusicalsymbols",
        "canadiansyllabics": "unifiedcanadianaboriginalsyllabics",
        "cherokeesup": "cherokeesupplement",
        "cjk": "cjkunifiedideographs",
        "cjkcompat": "cjkcompatibility",
        "cjkcompatforms": "cjkcompatibilityforms",
        "cjkcompatideographs": "cjkcompatibilityideographs",
        "cjkcompatideographssup": "cjkcompatibilityideographssupplement",
        "cjkexta": "cjkunifiedideographsextensiona",
        "cjkextb": "cjkunifiedideographsextensionb",
        "cjkextc": "cjkunifiedideographsextensionc",
        "cjkextd": "cjkunifiedideographsextensiond",
        "cjkexte": "cjkunifiedideographsextensione",
        "cjkextf": "cjkunifiedideographsextensionf",
        "cjkextg": "cjkunifiedideographsextensiong",
        "cjkexth": "cjkunifiedideographsextensionh",
        "cjkexti": "cjkunifiedideographsextensioni",
        "cjkradicalssup": "cjkradicalssupplement",
        "cjksymbols": "cjksymbolsandpunctuation",
        "combiningmarksforsymbols": "combiningdiacriticalmarksforsymbols",
        "compatjamo": "hangulcompatibilityjamo",
        "countingrod": "countingrodnumerals",
        "cuneiformnumbers": "cuneiformnumbersandpunctuation",
        "cyrillicexta": "cyrillicextendeda",
        "cyrillicextb": "cyrillicextendedb",
        "cyrillicextc": "cyrillicextendedc",
        "cyrillicextd": "cyrillicextendedd",
        "cyrillicsup": "cyrillicsupplement",
        "cyrillicsupplementary": "cyrillicsupplement",
        "devanagariext": "devanagariextended",
        "devanagariexta": "devanagariextendeda",
        "diacriticals": "combiningdiacriticalmarks",
        "diacriticalsext": "combiningdiacriticalmarksextended",
        "diacriticalsforsymbols": "combiningdiacriticalmarksforsymbols",
        "diacriticalssup": "combiningdiacriticalmarkssupplement",
        "domino": "dominotiles",
        "egyptianhieroglyphsexta": "egyptianhieroglyphsextendeda",
        "enclosedalphanum": "enclosedalphanumerics",
        "enclosedalphanumsup": "enclosedalphanumericsupplement",
        "enclosedcjk": "enclosedcjklettersandmonths",
        "enclosedideographicsup": "enclosedideographicsupplement",
        "ethiopicext": "ethiopicextended",
        "ethiopicexta": "ethiopicextendeda",
        "ethiopicextb": "ethiopicextendedb",
        "ethiopicsup": "ethiopicsupplement",
        "geometricshapesext": "geometricshapesextended",
        "georgianext": "georgianextended",
        "georgiansup": "georgiansupplement",
        "glagoliticsup": "glagoliticsupplement",
        "greek": "greekandcoptic",
        "greekext": "greekextended",
        "halfandfullforms": "halfwidthandfullwidthforms",
        "halfmarks": "combininghalfmarks",
        "hangul": "hangulsyllables",
        "highpusurrogates": "highprivateusesurrogates",
        "idc": "ideographicdescriptioncharacters",
        "ideographicsymbols": "ideographicsymbolsandpunctuation",
        "indicnumberforms": "commonindicnumberforms",
        "ipaext": "ipaextensions",
        "jamo": "hanguljamo",
        "jamoexta": "hanguljamoextendeda",
        "jamoextb": "hanguljamoextendedb",
        "kanaexta": "kanaextendeda",
        "kanaextb": "kanaextendedb",
        "kanasup": "kanasupplement",
        "kangxi": "kangxiradicals",
        "katakanaext": "katakanaphoneticextensions",
        "latin1": "latin1supplement",
        "latin1sup": "latin1supplement",
        "latinexta": "latinextendeda",
        "latinextadditional": "latinextendedadditional",
        "latinextb": "latinextendedb",
        "latinextc": "latinextendedc",
        "latinextd": "latinextendedd",
        "latinexte": "latinextendede",
        "latinextf": "latinextendedf",
        "latinextg": "latinextendedg",
        "lisusup": "lisusupplement",
        "mahjong": "mahjongtiles",
        "mathalphanum": "mathematicalalphanumericsymbols",
        "mathoperators": "mathematicaloperators",
        "meeteimayekext": "meeteimayekextensions",
        "miscarrows": "miscellaneoussymbolsandarrows",
        "miscmathsymbolsa": "miscellaneousmathematicalsymbolsa",
        "miscmathsymbolsb": "miscellaneousmathematicalsymbolsb",
        "miscpictographs": "miscellaneoussymbolsandpictographs",
        "miscsymbols": "miscellaneoussymbols",
        "misctechnical": "miscellaneoustechnical",
        "modifierletters": "spacingmodifierletters",
        "mongoliansup": "mongoliansupplement",
        "music": "musicalsymbols",
        "myanmarexta": "myanmarextendeda",
        "myanmarextb": "myanmarextendedb",
        "myanmarextc": "myanmarextendedc",
        "nb": "noblock",
        "ocr": "opticalcharacterrecognition",
        "phaistos": "phaistosdisc",
        "phoneticext": "phoneticextensions",
        "phoneticextsup": "phoneticextensionssupplement",
        "posixascii": "basiclatin",
        "privateuse": "privateusearea",
        "pua": "privateusearea",
        "punctuation": "generalpunctuation",
        "rumi": "ruminumeralsymbols",
        "smallforms": "smallformvariants",
        "smallkanaext": "smallkanaextension",
        "sundanesesup": "sundanesesupplement",
        "suparrowsa": "supplementalarrowsa",
        "suparrowsb": "supplementalarrowsb",
        "suparrowsc": "supplementalarrowsc",
        "superandsub": "superscriptsandsubscripts",
        "supmathoperators": "supplementalmathematicaloperators",
        "suppuaa": "supplementaryprivateuseareaa",
        "suppuab": "supplementaryprivateuseareab",
        "suppunctuation": "supplementalpunctuation",
        "supsymbolsandpictographs": "supplementalsymbolsandpictographs",
        "symbolsandpictographsexta": "symbolsandpictographsextendeda",
        "symbolsforlegacycomputingsup": "symbolsforlegacycomputingsupplement",
        "syriacsup": "syriacsupplement",
        "taixuanjing": "taixuanjingsymbols",
        "tamilsup": "tamilsupplement",
        "tangutsup": "tangutsupplement",
        "transportandmap": "transportandmapsymbols",
        "ucas": "unifiedcanadianaboriginalsyllabics",
        "ucasext": "unifiedcanadianaboriginalsyllabicsextended",
        "ucasexta": "unifiedcanadianaboriginalsyllabicsextendeda",
        "vedicext": "vedicextensions",
        "vs": "variationselectors",
        "vssup": "variationselectorssupplement",
        "yijing": "yijinghexagramsymbols",
        "znamennymusic": "znamennymusicalnotation"
    },
    "canonicalcombiningclass": {
        "a": "230",
        "above": "230",
        "aboveleft": "228",
        "aboveright": "232",
        "al": "228",
        "ar": "232",
        "ata": "214",
        "atar": "216",
        "atb": "202",
        "atbl": "200",
        "attachedabove": "214",
        "attachedaboveright": "216",
        "attachedbelow": "202",
        "attachedbelowleft": "200",
        "b": "220",
        "below": "220",
        "belowleft": "218",
        "belowright": "222",
        "bl": "218",
        "br": "222",
        "ccc10": "10",
        "ccc103": "103",
        "ccc107": "107",
        "ccc11": "11",
        "ccc118": "118",
        "ccc12": "12",
        "ccc122": "122",
        "ccc129": "129",
        "ccc13": "13",
        "ccc130": "130",
        "ccc132": "132",
        "ccc133": "133",
        "ccc14": "14",
        "ccc15": "15",
        "ccc16": "16",
        "ccc17": "17",
        "ccc18": "18",
        "ccc19": "19",
        "ccc20": "20",
        "ccc21": "21",
        "ccc22": "22",
        "ccc23": "23",
        "ccc24": "24",
        "ccc25": "25",
        "ccc26": "26",
        "ccc27": "27",
        "ccc28": "28",
        "ccc29": "29",
        "ccc30": "30",
        "ccc31": "31",
        "ccc32": "32",
        "ccc33": "33",
        "ccc34": "34",
        "ccc35": "35",
        "ccc36": "36",
        "ccc84": "84",
        "ccc91": "91",
        "da": "234",
        "db": "233",
        "doubleabove": "234",
        "doublebelow": "233",
        "hanr": "6",
        "hanreading": "6",
        "iotasubscript": "240",
        "is": "240",
        "kanavoicing": "8",
        "kv": "8",
        "l": "224",
        "left": "224",
        "nk": "7",
        "notreordered": "0",
        "nr": "0",
        "nukta": "7",
        "ov": "1",
        "overlay": "1",
        "r": "226",
        "right": "226",
        "virama": "9",
        "vr": "9"
    },
    "decompositiontype": {
        "can": "canonical",
        "com": "compat",
        "enc": "circle",
        "fin": "final",
        "fra": "fraction",
        "init": "initial",
        "iso": "isolated",
        "med": "medial",
        "nar": "narrow",
        "nb": "nobreak",
        "sml": "small",
        "sqr": "square",
        "sup": "super",
        "vert": "vertical"
    },
    "eastasianwidth": {
        "ambiguous": "a",
        "fullwidth": "f",
        "halfwidth": "h",
        "narrow": "na",
        "neutral": "n",
        "wide": "w"
    },
    "generalcategory": {
        "casedletter": "lc",
        "closepunctuation": "pe",
        "cntrl": "cc",
        "combiningmark": "m",
        "connectorpunctuation": "pc",
        "control": "cc",
        "currencysymbol": "sc",
        "dashpunctuation": "pd",
        "decimalnumber": "nd",
        "digit": "nd",
        "enclosingmark": "me",
        "finalpunctuation": "pf",
        "format": "cf",
        "initialpunctuation": "pi",
        "letter": "l",
        "letternumber": "nl",
        "lineseparator": "zl",
        "lowercaseletter": "ll",
        "mark": "m",
        "mathsymbol": "sm",
        "modifierletter": "lm",
        "modifiersymbol": "sk",
        "nonspacingmark": "mn",
        "number": "n",
        "openpunctuation": "ps",
        "other": "c",
        "otherletter": "lo",
        "othernumber": "no",
        "otherpunctuation": "po",
        "othersymbol": "so",
        "paragraphseparator": "zp",
        "privateuse": "co",
        "punct": "p",
        "punctuation": "p",
        "separator": "z",
        "spaceseparator": "zs",
        "spacingmark": "mc",
        "surrogate": "cs",
        "symbol": "s",
        "titlecaseletter": "lt",
        "unassigned": "cn",
        "uppercaseletter": "lu"
    },
    "graphemeclusterbreak": {
        "cn": "control",
        "eb": "ebase",
        "ebg": "ebasegaz",
        "em": "emodifier",
        "ex": "extend",
        "gaz": "glueafterzwj",
        "pp": "prepend",
        "ri": "regionalindicator",
        "sm": "spacingmark",
        "xx": "other"
    },
    "hangulsyllabletype": {
        "leadingjamo": "l",
        "lvsyllable": "lv",
        "lvtsyllable": "lvt",
        "notapplicable": "na",
        "trailingjamo": "t",
        "voweljamo": "v"
    },
    "indicpositionalcategory": {
    },
    "indicsyllabiccategory": {
    },
    "joininggroup": {
        "hamzaonhehgoal": "tehmarbutagoal"
    },
    "joiningtype": {
        "dualjoining": "d",
        "joincausing": "c",
        "leftjoining": "l",
        "nonjoining": "u",
        "rightjoining": "r",
        "transparent": "t"
    },
    "linebreak": {
        "aksara": "ak",
        "aksaraprebase": "ap",
        "aksarastart": "as",
        "alphabetic": "al",
        "ambiguous": "ai",
        "breakafter": "ba",
        "breakbefore": "bb",
        "breakboth": "b2",
        "breaksymbols": "sy",
        "carriagereturn": "cr",
        "closeparenthesis": "cp",
        "closepunctuation": "cl",
        "combiningmark": "cm",
        "complexcontext": "sa",
        "conditionaljapanesestarter": "cj",
        "contingentbreak": "cb",
        "ebase": "eb",
        "emodifier": "em",
        "exclamation": "ex",
        "glue": "gl",
        "hebrewletter": "hl",
        "hyphen": "hy",
        "ideographic": "id",
        "infixnumeric": "is",
        "inseparable": "in",
        "inseperable": "in",
        "linefeed": "lf",
        "mandatorybreak": "bk",
        "nextline": "nl",
        "nonstarter": "ns",
        "numeric": "nu",
        "openpunctuation": "op",
        "postfixnumeric": "po",
        "prefixnumeric": "pr",
        "quotation": "qu",
        "regionalindicator": "ri",
        "space": "sp",
        "surrogate": "sg",
        "unknown": "xx",
        "virama": "vi",
        "viramafinal": "vf",
        "wordjoiner": "wj",
        "zwspace": "zw"
    },
    "nfcquickcheck": {
        "maybe": "m",
        "no": "n",
        "yes": "y"
    },
    "nfdquickcheck": {
        "no": "n",
        "yes": "y"
    },
    "nfkcquickcheck": {
        "maybe": "m",
        "no": "n",
        "yes": "y"
    },
    "nfkdquickcheck": {
        "no": "n",
        "yes": "y"
    },
    "numerictype": {
        "de": "decimal",
        "di": "digit",
        "nu": "numeric"
    },
    "numericvalue": {
    },
    "script": {
        "adlm": "adlam",
        "aghb": "caucasianalbanian",
        "arab": "arabic",
        "armi": "imperialaramaic",
        "armn": "armenian",
        "avst": "avestan",
        "bali": "balinese",
        "bamu": "bamum",
        "bass": "bassavah",
        "batk": "batak",
        "beng": "bengali",
        "bhks": "bhaiksuki",
        "bopo": "bopomofo",
        "brah": "brahmi",
        "brai": "braille",
        "bugi": "buginese",
        "buhd": "buhid",
        "cakm": "chakma",
        "cans": "canadianaboriginal",
        "cari": "carian",
        "cher": "cherokee",
        "chrs": "chorasmian",
        "copt": "coptic",
        "cpmn": "cyprominoan",
        "cprt": "cypriot",
        "cyrl": "cyrillic",
        "deva": "devanagari",
        "diak": "divesakuru",
        "dogr": "dogra",
        "dsrt": "deseret",
        "dupl": "duployan",
        "egyp": "egyptianhieroglyphs",
        "elba": "elbasan",
        "elym": "elymaic",
        "ethi": "ethiopic",
        "gara": "garay",
        "geor": "georgian",
        "glag": "glagolitic",
        "gong": "gunjalagondi",
        "gonm": "masaramgondi",
        "goth": "gothic",
        "gran": "grantha",
        "grek": "greek",
        "gujr": "gujarati",
        "gukh": "gurungkhema",
        "guru": "gurmukhi",
        "hang": "hangul",
        "hani": "han",
        "hano": "hanunoo",
        "hatr": "hatran",
        "hebr": "hebrew",
        "hira": "hiragana",
        "hluw": "anatolianhieroglyphs",
        "hmng": "pahawhhmong",
        "hmnp": "nyiakengpuachuehmong",
        "hrkt": "katakanaorhiragana",
        "hung": "oldhungarian",
        "ital": "olditalic",
        "java": "javanese",
        "kali": "kayahli",
        "kana": "katakana",
        "khar": "kharoshthi",
        "khmr": "khmer",
        "khoj": "khojki",
        "kits": "khitansmallscript",
        "knda": "kannada",
        "krai": "kiratrai",
        "kthi": "kaithi",
        "lana": "taitham",
        "laoo": "lao",
        "latn": "latin",
        "lepc": "lepcha",
        "limb": "limbu",
        "lina": "lineara",
        "linb": "linearb",
        "lyci": "lycian",
        "lydi": "lydian",
        "mahj": "mahajani",
        "maka": "makasar",
        "mand": "mandaic",
        "mani": "manichaean",
        "marc": "marchen",
        "medf": "medefaidrin",
        "mend": "mendekikakui",
        "merc": "meroiticcursive",
        "mero": "meroitichieroglyphs",
        "mlym": "malayalam",
        "mong": "mongolian",
        "mroo": "mro",
        "mtei": "meeteimayek",
        "mult": "multani",
        "mymr": "myanmar",
        "nagm": "nagmundari",
        "nand": "nandinagari",
        "narb": "oldnortharabian",
        "nbat": "nabataean",
        "nkoo": "nko",
        "nshu": "nushu",
        "ogam": "ogham",
        "olck": "olchiki",
        "onao": "olonal",
        "orkh": "oldturkic",
        "orya": "oriya",
        "osge": "osage",
        "osma": "osmanya",
        "ougr": "olduyghur",
        "palm": "palmyrene",
        "pauc": "paucinhau",
        "perm": "oldpermic",
        "phag": "phagspa",
        "phli": "inscriptionalpahlavi",
        "phlp": "psalterpahlavi",
        "phnx": "phoenician",
        "plrd": "miao",
        "prti": "inscriptionalparthian",
        "qaac": "coptic",
        "qaai": "inherited",
        "rjng": "rejang",
        "rohg": "hanifirohingya",
        "runr": "runic",
        "samr": "samaritan",
        "sarb": "oldsoutharabian",
        "saur": "saurashtra",
        "sgnw": "signwriting",
        "shaw": "shavian",
        "shrd": "sharada",
        "sidd": "siddham",
        "sind": "khudawadi",
        "sinh": "sinhala",
        "sogd": "sogdian",
        "sogo": "oldsogdian",
        "sora": "sorasompeng",
        "soyo": "soyombo",
        "sund": "sundanese",
        "sunu": "sunuwar",
        "sylo": "sylotinagri",
        "syrc": "syriac",
        "tagb": "tagbanwa",
        "takr": "takri",
        "tale": "taile",
        "talu": "newtailue",
        "taml": "tamil",
        "tang": "tangut",
        "tavt": "taiviet",
        "telu": "telugu",
        "tfng": "tifinagh",
        "tglg": "tagalog",
        "thaa": "thaana",
        "tibt": "tibetan",
        "tirh": "tirhuta",
        "tnsa": "tangsa",
        "todr": "todhri",
        "tutg": "tulutigalari",
        "ugar": "ugaritic",
        "vaii": "vai",
        "vith": "vithkuqi",
        "wara": "warangciti",
        "wcho": "wancho",
        "xpeo": "oldpersian",
        "xsux": "cuneiform",
        "yezi": "yezidi",
        "yiii": "yi",
        "zanb": "zanabazarsquare",
        "zinh": "inherited",
        "zyyy": "common",
        "zzzz": "unknown"
    },
    "scriptextensions": {
    },
    "sentencebreak": {
        "at": "aterm",
        "cl": "close",
        "ex": "extend",
        "fo": "format",
        "le": "oletter",
        "lo": "lower",
        "nu": "numeric",
        "sc": "scontinue",
        "se": "sep",
        "st": "sterm",
        "up": "upper",
        "xx": "other"
    },
    "verticalorientation": {
        "rotated": "r",
        "transformedrotated": "tr",
        "transformedupright": "tu",
        "upright": "u"
    },
    "wordbreak": {
        "dq": "doublequote",
        "eb": "ebase",
        "ebg": "ebasegaz",
        "em": "emodifier",
        "ex": "extendnumlet",
        "fo": "format",
        "gaz": "glueafterzwj",
        "hl": "hebrewletter",
        "ka": "katakana",
        "le": "aletter",
        "mb": "midnumlet",
        "ml": "midletter",
        "mn": "midnum",
        "nl": "newline",
        "nu": "numeric",
        "ri": "regionalindicator",
        "sq": "singlequote",
        "xx": "other"
    }
}
